import BottomNavigation from '@mui/material/BottomNavigation';
import BottomNavigationAction from '@mui/material/BottomNavigationAction';
import { useState } from 'react';
import { useSelector } from 'react-redux';
import { makeStyles } from '@material-ui/core'

import AudioList from './AudioList';
import MediaList from './MediaList';
import FilesList from './FilesList';
import TextList from './TextList';
import { getMessagesMemo } from '../../../../../../../redux/messages/selector'

const useStyles = makeStyles({
    bottomNavigation: {
        boxShadow: '0px 1px 1px 1px rgba(120,120,120,0.63)',
        marginBottom: 20
    },
    icon: {
        marginBottom:0,
    },
    underline: {
        fontSize: 30,
        lineHeight:0
    },
})

const ProfileLists = () => {
    const classes = useStyles()
    const messagesMemo = useSelector(getMessagesMemo)
    const [isActive, setIsActive] = useState<number>(0)
    const handleIsActive = (_e: React.SyntheticEvent<Element, Event>, newValue: number): void => setIsActive(newValue)
    const Icon = ({ name }: { name: string }) => <span className={classes.icon}>{name}</span>
    const Label = () => <span className={classes.underline}>__</span>
    return (
    <>
      <BottomNavigation
        showLabels
        value={isActive}
        onChange={handleIsActive}
        className={classes.bottomNavigation}
      >
            <BottomNavigationAction label={<Label/>} icon={<Icon name='Media' />} />
            <BottomNavigationAction label={<Label/>} icon={<Icon name='Files' />} />
            <BottomNavigationAction label={<Label/>} icon={<Icon name='Text' />} />
            <BottomNavigationAction label={<Label/>} icon={<Icon name='Music' />} />

        </BottomNavigation>
            {isActive === 0 && <MediaList messagesMemo={messagesMemo}/>}
            {isActive === 1 && <FilesList messagesMemo={messagesMemo}/>}
            {isActive === 2 && <TextList messagesMemo={messagesMemo}/>}
            {isActive === 3 && <AudioList messagesMemo={messagesMemo}/>}
    </>      
    )
}

export default ProfileLists